@@ -17,6 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include "xml-dir.h"
 #include "xml-entry.h"
 
@@ -33,7 +34,7 @@
 #include <errno.h>
 #include <limits.h>
 
-#include <gconf/gconf-internals.h>
+#include "gconf/gconf-internals.h"
 
 /* This makes hash table safer when debugging */
 #ifndef GCONF_ENABLE_DEBUG
@@ -956,12 +957,6 @@ dir_set_schema  (Dir         *d,
     dir_forget_entry_if_useless (d, e);
 }
 
-GTime
-dir_last_access (Dir* d)
-{
-  return d->last_access;
-}
-
 /* private Dir functions */
 
 static void
@@ -1251,8 +1246,7 @@ create_fs_dir(const gchar* dir, const gchar* xml_filename,
             gconf_log(GCL_DEBUG, "failed parent: %s", parent);
           
           g_free(parent);
-          if (parent_xml)
-            g_free(parent_xml);
+          g_free(parent_xml);
           
           if (!success)
             return FALSE;
