@@ -22,9 +22,7 @@
 
 #include <glib.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 #include "gconf-error.h"
 #include "GConfX.h"
@@ -70,11 +68,12 @@ CORBA_unsigned_long gconf_database_readd_listener   (GConfDatabase       *db,
                                                      const gchar         *where);
 
 void                gconf_database_notify_listeners (GConfDatabase       *db,
+                                                     GConfSources        *modified_sources,
                                                      const gchar         *key,
                                                      const ConfigValue   *value,
                                                      gboolean             is_default,
-                                                     gboolean             is_writable);
-
+                                                     gboolean             is_writable,
+                                                     gboolean             notify_others);
 
 GConfValue* gconf_database_query_value         (GConfDatabase  *db,
                                                 const gchar    *key,
@@ -134,6 +133,9 @@ gboolean gconf_database_synchronous_sync (GConfDatabase  *db,
                                           GError    **err);
 void     gconf_database_clear_cache      (GConfDatabase  *db,
                                           GError    **err);
+void     gconf_database_clear_cache_for_sources (GConfDatabase  *db,
+						 GConfSources   *sources,
+						 GError        **err);
 
 
 void gconfd_locale_cache_expire (void);
@@ -145,9 +147,7 @@ void gconf_database_log_listeners_to_string (GConfDatabase *db,
                                              gboolean is_default,
                                              GString *str);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif
 
