@@ -22,14 +22,12 @@
 
 #include <glib.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
-#include <gconf/gconf-schema.h>
-#include <gconf/gconf-engine.h>
-#include <gconf/gconf-error.h>
-#include <gconf/gconf-enum-types.h>
+#include "gconf/gconf-schema.h"
+#include "gconf/gconf-engine.h"
+#include "gconf/gconf-error.h"
+#include "gconf/gconf-enum-types.h"
 
 typedef void (*GConfNotifyFunc) (GConfEngine* conf,
                                  guint cnxn_id,
@@ -304,8 +302,6 @@ GConfValue * gconf_engine_get_full (GConfEngine *conf,
 
 #endif /* GCONF_ENABLE_INTERNALS */
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif
