@@ -49,15 +49,14 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <ctype.h>
-#ifdef HAVE_SYSLOG_H
-#include <syslog.h>
-#endif
 #include <time.h>
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #endif
 #include <locale.h>
 
+#include <dbus/dbus-glib-lowlevel.h>
+
 /* This makes hash table safer when debugging */
 #ifndef GCONF_ENABLE_DEBUG
 #define safe_g_hash_table_insert g_hash_table_insert
@@ -500,112 +499,235 @@ gconf_get_poa (void)
   return the_poa;
 }
 
-static void
-log_handler (const gchar   *log_domain,
-             GLogLevelFlags log_level,
-             const gchar   *message,
-             gpointer       user_data)
+static const char *
+get_introspection_xml (void)
 {
-  GConfLogPriority pri = GCL_WARNING;
-  
-  switch (log_level & G_LOG_LEVEL_MASK)
-    {
-    case G_LOG_LEVEL_ERROR:
-    case G_LOG_LEVEL_CRITICAL:
-      pri = GCL_ERR;
-      break;
-
-    case G_LOG_LEVEL_WARNING:
-      pri = GCL_WARNING;
-      break;
-
-    case G_LOG_LEVEL_MESSAGE:
-    case G_LOG_LEVEL_INFO:
-      pri = GCL_INFO;
-      break;
-
-    case G_LOG_LEVEL_DEBUG:
-      pri = GCL_DEBUG;
-      break;
-
-    default:
-      break;
-    }
-
-  gconf_log (pri, "%s", message);
+  return "<!DOCTYPE node PUBLIC \"-//freedesktop//DTD D-BUS Object Introspection 1.0//EN\"\n"
+         "\"http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd\">\n"
+         "<node>\n"
+         "  <interface name=\"org.freedesktop.DBus.Introspectable\">\n"
+         "    <method name=\"Introspect\">\n"
+         "      <arg name=\"introspection_xml\" direction=\"out\" type=\"s\"/>\n"
+         "    </method>\n"
+         "  </interface>\n"
+         "  <interface name=\"org.gnome.GConf\">\n"
+         "    <method name=\"GetIOR\">\n"
+         "      <arg name=\"ior\" direction=\"out\" type=\"s\"/>\n"
+         "    </method>\n"
+         "  </interface>\n"
+         "</node>\n";
 }
 
-/* From ORBit2 */
-/* There is a DOS attack if another user creates
- * the given directory and keeps us from creating
- * it
- */
-static gboolean
-test_safe_tmp_dir (const char *dirname)
+static DBusHandlerResult
+bus_message_handler (DBusConnection *connection,
+                     DBusMessage    *message,
+                     void           *user_data)
 {
-#ifndef G_OS_WIN32
-  struct stat statbuf;
-  int fd;
+  DBusMessage *reply;
 
-  fd = open (dirname, O_RDONLY);  
-  if (fd < 0)
-    {
-      gconf_log (GCL_WARNING, _("Failed to open %s: %s"),
-                 dirname, g_strerror (errno));
-      return FALSE;
-    }
-  
-  if (fstat (fd, &statbuf) != 0)
-    {
-      gconf_log (GCL_WARNING, _("Failed to stat %s: %s"),
-                 dirname, g_strerror (errno));
-      close (fd);
-      return FALSE;
-    }
-  close (fd);
+  reply = NULL;
 
-  if (statbuf.st_uid != getuid ())
+  if (dbus_message_is_signal (message,
+                              DBUS_INTERFACE_LOCAL,
+                              "Disconnected"))
     {
-      gconf_log (GCL_WARNING, _("Owner of %s is not the current user"),
-                 dirname);
-      return FALSE;
+      gconf_main_quit ();
+      return DBUS_HANDLER_RESULT_HANDLED;
+    }
+  else if (dbus_message_is_method_call (message,
+                                        "org.freedesktop.DBus.Introspectable",
+                                        "Introspect"))
+    {
+      const char *introspection_xml;
+
+      introspection_xml = get_introspection_xml ();
+
+      reply = dbus_message_new_method_return (message);
+      dbus_message_append_args (reply, DBUS_TYPE_STRING, &introspection_xml,
+                                DBUS_TYPE_INVALID);
+
+    }
+  else if (dbus_message_is_method_call (message,
+                                        "org.gnome.GConf",
+                                        "GetIOR"))
+    {
+      const char *ior;
+
+      ior = gconf_get_daemon_ior ();
+
+      reply = dbus_message_new_method_return (message);
+      dbus_message_append_args (reply, DBUS_TYPE_STRING, &ior, DBUS_TYPE_INVALID);
     }
 
-  if ((statbuf.st_mode & (S_IRWXG|S_IRWXO)) ||
-      !S_ISDIR (statbuf.st_mode))
+  if (reply != NULL)
     {
-      gconf_log (GCL_WARNING, _("Bad permissions %lo on directory %s"),
-                 (unsigned long) statbuf.st_mode & 07777, dirname);
-      return FALSE;
+      dbus_connection_send (connection, reply, NULL);
+      dbus_message_unref (reply);
+      return DBUS_HANDLER_RESULT_HANDLED;
     }
-#else
-  /* FIXME: We can't get any useful information about the actual
-   * protection for the directory using stat(). We must use the Win32
-   * API to check the owner and permissions (ACL). Later.
-   */
-#endif
-  
-  return TRUE;
+
+  return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
+static DBusConnection *
+get_on_d_bus (void)
+{
+  DBusConnection *connection;
+  DBusError bus_error;
+  int result;
+
+  dbus_error_init (&bus_error);
+  connection = dbus_bus_get (DBUS_BUS_SESSION, &bus_error);
+
+  if (dbus_error_is_set (&bus_error))
+    {
+      gconf_log (GCL_ERR, _("Could not connect to session bus: %s"), bus_error.message);
+      dbus_error_free (&bus_error);
+      return NULL;
+    }
+
+  dbus_connection_setup_with_g_main (connection, NULL);
+
+  if (!dbus_connection_add_filter (connection, (DBusHandleMessageFunction)
+                                  bus_message_handler, NULL, NULL))
+    {
+      dbus_connection_unref (connection);
+      return NULL;
+    }
+
+  dbus_connection_set_exit_on_disconnect (connection, FALSE);
+
+  result = dbus_bus_request_name (connection, "org.gnome.GConf",
+                                  0, &bus_error);
+
+  if (dbus_error_is_set (&bus_error))
+    {
+      gconf_log (GCL_WARNING,
+                 _("Failed to get bus name for daemon, exiting: %s"),
+                 bus_error.message);
+      dbus_error_free (&bus_error);
+    }
+
+  if (result != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER)
+    {
+      dbus_connection_unref (connection);
+      return NULL;
+    }
+
+  return connection;
+}
+
+#ifdef ENABLE_DEFAULTS_SERVICE
+/* listen on system bus for defaults changes */
+
+static DBusHandlerResult
+system_bus_message_handler (DBusConnection *connection,
+			    DBusMessage    *message,
+			    void           *user_data)
+{
+  DBusMessage *reply;
+
+  reply = NULL;
+
+  if (dbus_message_is_signal (message,
+			      "org.gnome.GConf.Defaults",
+                              "SystemSet"))
+    {
+      DBusError bus_error;
+      char **keys;
+      int n_keys;
+
+      dbus_error_init (&bus_error);
+      if (dbus_message_get_args (message, &bus_error,
+				 DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &keys, &n_keys,
+				 DBUS_TYPE_INVALID))
+	{
+	  char **key;
+	  GConfSources *system_sources;
+	  GSList addresses;
+
+	  gconf_log (GCL_DEBUG, "System defaults changed.  Notifying.");
+
+	  addresses.data = "xml:merged:/etc/gconf/gconf.xml.system";
+	  addresses.next = NULL;
+	  system_sources = gconf_sources_new_from_addresses (&addresses, NULL);
+
+	  gconfd_clear_cache_for_sources (system_sources);
+
+	  for (key = keys; *key; key++)
+	    gconfd_notify_other_listeners (NULL, system_sources, *key);
+
+	  gconf_sources_free (system_sources);
+
+	  dbus_free_string_array (keys);
+	}
+      else
+        {
+	  gconf_log (GCL_DEBUG, "SystemSet signal received, but error getting message: %s", bus_error.message);
+	}
+      dbus_error_free (&bus_error);
+
+      return DBUS_HANDLER_RESULT_HANDLED;
+    }
+
+  return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
+
+static DBusConnection *
+get_on_system_bus (void)
+{
+  DBusConnection *connection;
+  DBusError bus_error;
+
+  dbus_error_init (&bus_error);
+  connection = dbus_bus_get (DBUS_BUS_SYSTEM, &bus_error);
+
+  if (dbus_error_is_set (&bus_error))
+    {
+      gconf_log (GCL_ERR, _("Could not connect to system bus: %s"), bus_error.message);
+      dbus_error_free (&bus_error);
+      return NULL;
+    }
+
+  dbus_connection_setup_with_g_main (connection, NULL);
+
+  dbus_bus_add_match (connection, "type='signal',interface='org.gnome.GConf.Defaults'", &bus_error);
+  dbus_connection_flush(connection);
+  if (dbus_error_is_set (&bus_error))
+    {
+      gconf_log (GCL_DEBUG, "Failed to add signal match to system bus: %s", bus_error.message);
+      dbus_connection_unref (connection);
+      return NULL;
+    }
+
+  if (!dbus_connection_add_filter (connection, (DBusHandleMessageFunction)
+                                   system_bus_message_handler, NULL, NULL))
+    {
+      gconf_log (GCL_DEBUG, "Failed to add message filter to system bus.");
+      dbus_connection_unref (connection);
+      return NULL;
+    }
+
+  return connection;
+}
+#endif  /* ENABLE_DEFAULTS_SERVICE */
+
 int 
 main(int argc, char** argv)
 {
 #ifdef HAVE_SIGACTION
   struct sigaction act;
   sigset_t empty_mask;
+  sigset_t full_mask;
 #endif
   CORBA_Environment ev;
   CORBA_ORB orb;
-  gchar* logname;
-  const gchar* username;
   gchar* ior;
   int exit_code = 0;
   GError *err;
-  char *lock_dir;
-  char *gconfd_dir;
   int dev_null_fd;
   int write_byte_fd;
+  DBusConnection *connection;
 
   _gconf_init_i18n ();
   setlocale (LC_ALL, "");
@@ -620,7 +742,7 @@ main(int argc, char** argv)
   /* This is so we don't prevent unmounting of devices. We divert
    * all messages to syslog
    */
-  if (chdir ("/") < 0)
+  if (g_chdir ("/") < 0)
     {
        g_printerr ("Could not change to root directory: %s\n",
 		   g_strerror (errno));
@@ -646,27 +768,7 @@ main(int argc, char** argv)
   
   gconf_set_daemon_mode(TRUE);
   
-  /* Logs */
-  username = g_get_user_name();
-  logname = g_strdup_printf("gconfd (%s-%u)", username, (guint)getpid());
-
-#ifdef HAVE_SYSLOG_H
-  openlog (logname, LOG_NDELAY, LOG_USER);
-#endif
-
-  g_log_set_handler (NULL, G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
-                     log_handler, NULL);
-
-  g_log_set_handler ("GLib", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
-                     log_handler, NULL);
-
-  g_log_set_handler ("GLib-GObject", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
-                     log_handler, NULL);
-  
-  /* openlog() does not copy logname - what total brokenness.
-     So we free it at the end of main() */
-  
-  gconf_log (GCL_INFO, _("starting (version %s), pid %u user '%s'"), 
+  gconf_log (GCL_DEBUG, _("starting (version %s), pid %u user '%s'"), 
              VERSION, (guint)getpid(), g_get_user_name());
 
 #ifdef GCONF_ENABLE_DEBUG
@@ -675,6 +777,9 @@ main(int argc, char** argv)
   
   /* Session setup */
 #ifdef HAVE_SIGACTION
+  sigfillset (&full_mask);
+  sigprocmask (SIG_UNBLOCK, &full_mask, NULL);
+
   sigemptyset (&empty_mask);
   act.sa_handler = signal_handler;
   act.sa_mask    = empty_mask;
@@ -732,32 +837,9 @@ main(int argc, char** argv)
   gconf_set_daemon_ior (ior);
   CORBA_free (ior);
 
-  gconfd_dir = gconf_get_daemon_dir ();
-  lock_dir = gconf_get_lock_dir ();
-  
-  if (g_mkdir (gconfd_dir, 0700) < 0 && errno != EEXIST)
-    gconf_log (GCL_WARNING, _("Failed to create %s: %s"),
-               gconfd_dir, g_strerror (errno));
-  
-  if (!test_safe_tmp_dir (gconfd_dir))
-    {
-      err = g_error_new (GCONF_ERROR,
-                         GCONF_ERROR_LOCK_FAILED,
-                         _("Directory %s has a problem, gconfd can't use it"),
-                         gconfd_dir);
-      daemon_lock = NULL;
-    }
-  else
-    {
-      err = NULL;
-      
-      daemon_lock = gconf_get_lock (lock_dir, &err);
-    }
+  connection = get_on_d_bus ();
 
-  g_free (gconfd_dir);
-  g_free (lock_dir);
-
-  if (daemon_lock != NULL)
+  if (connection != NULL)
     {
       /* This loads backends and so on. It needs to be done before
        * we can handle any requests, so before we hit the
@@ -781,14 +863,8 @@ main(int argc, char** argv)
       close (write_byte_fd);
     }
   
-  if (daemon_lock == NULL)
+  if (connection == NULL)
     {
-      g_assert (err);
-
-      gconf_log (GCL_WARNING, _("Failed to get lock for daemon, exiting: %s"),
-                 err->message);
-      g_error_free (err);
-
       enter_shutdown ();
       shutdown_databases ();
       
@@ -797,7 +873,11 @@ main(int argc, char** argv)
 
   /* Read saved log file, if any */
   logfile_read ();
-  
+ 
+#ifdef ENABLE_DEFAULTS_SERVICE 
+  get_on_system_bus ();
+#endif
+
   gconf_main ();
 
   if (in_shutdown)
@@ -831,14 +911,7 @@ main(int argc, char** argv)
 
   daemon_lock = NULL;
   
-  gconf_log (GCL_INFO, _("Exiting"));
-
-#ifdef HAVE_SYSLOG_H
-  closelog ();
-#endif
-
-  /* Can't do this due to stupid atexit() handler that calls g_log stuff */
-  /*   g_free (logname); */
+  gconf_log (GCL_DEBUG, _("Exiting"));
 
   return exit_code;
 }
@@ -910,12 +983,12 @@ gconf_main(void)
 
   if (main_loops == NULL)
     {
-      gulong timeout_len = 1000*60*0.5; /* 1 sec * 60 s/min * .5 min */
+      gulong timeout_len = 60*0.5; /* 60 s/min * .5 min */
       
       g_assert(timeout_id == 0);
-      timeout_id = g_timeout_add (timeout_len,
-                                  periodic_cleanup_timeout,
-                                  NULL);
+      timeout_id = g_timeout_add_seconds (timeout_len,
+                                          periodic_cleanup_timeout,
+                                          NULL);
 
     }
   
@@ -1233,6 +1306,22 @@ gconfd_notify_other_listeners (GConfDatabase *modified_db,
     }
 }
 
+void
+gconfd_clear_cache_for_sources (GConfSources *sources)
+{
+  GList *tmp;
+
+  tmp = db_list;
+  while (tmp != NULL)
+    {
+      GConfDatabase *db = tmp->data;
+
+      gconf_database_clear_cache_for_sources (db, sources, NULL);
+
+      tmp = tmp->next;
+    }
+}
+
 /*
  * Cleanup
  */
@@ -1447,14 +1536,14 @@ open_append_handle (GError **err)
 
 
       {
-        const gulong timeout_len = 1000*60*0.5; /* 1 sec * 60 s/min * 0.5 min */
+        const gulong timeout_len = 60*0.5; /* 60 s/min * 0.5 min */
 
         if (append_handle_timeout != 0)
           g_source_remove (append_handle_timeout);
         
-        append_handle_timeout = g_timeout_add (timeout_len,
-                                               close_append_handle_timeout,
-                                               NULL);
+        append_handle_timeout = g_timeout_add_seconds (timeout_len,
+                                                       close_append_handle_timeout,
+                                                       NULL);
       }
     }
 
