@@ -22,9 +22,7 @@
 
 #include <glib.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 #include "gconf-error.h"
 #include "GConfX.h"
@@ -44,11 +42,16 @@ gboolean gconfd_logfile_change_listener (GConfDatabase *db,
 
 gboolean gconfd_check_in_shutdown (CORBA_Environment *ev);
 
+void gconfd_notify_other_listeners (GConfDatabase *modified_db,
+				    GConfSources  *modified_sources,
+                                    const char    *key);
+
+void
+gconfd_clear_cache_for_sources (GConfSources *sources);
+
 void gconfd_need_log_cleanup (void);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif
 
